package Commands is

   task type Handler_Task is
     entry Start;
     entry Stop;
   end Handler_Task;

   protected Repository is
      procedure Quit;
      function Have_To_Quit return Boolean;
   private
      Quit_Pressed : Boolean := False;
   end Repository;

end Commands;
